#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "kodak_dc240_jd.h"
#include <time.h>
#include <ctype.h>

char serial_port[20];

int getidfromname (char * name)
{
    int i;

    kdc240_number_of_pictures();

    for (i = 0; i < number_of_pictures; i++)
	if (strcmp(picture_index[i].filename, name) == 0)
	    return i + 1;

    fprintf (stderr, "Warning: could not find any match in the %d entries for %s\n",
	     number_of_pictures, name);
    return 0;
}

int main (int nb, char ** cmde)
{   int id;
    char *prefix = NULL,
         *tstamp = NULL;
    strcpy (serial_port, "/dev/usb/dc240");

    {	int i;
	int musthalt = 0;
	for (i=1 ; i<nb ; i++)
	{
	    if (cmde[i][0] == '-')
	    {	
		if ( (strcmp ("help", cmde[i]+1) == 0) || (strcmp ("h", cmde[i]+1) == 0))
		{   printf ("usage: %s "
			    "       [-p prefix | -prefix prefix]\n",
			    cmde[0]);
		    musthalt = 1;
		    continue;
		}

		if ( (strcmp ("prefix", cmde[i]+1) == 0) || (strcmp ("p", cmde[i]+1) == 0))
		{   if (i+1 >= nb)
		    {	fprintf (stderr, "missing data after flag %s\n", cmde[i]);
			musthalt = 1;
			continue;
		    }
		    if ((prefix = (char *) malloc (strlen (cmde[i+1]) + 30)) == NULL)
		    {	fprintf (stderr, "could not allocate %d byte for prefix !\n",
			         strlen (cmde[i+1]) + 30);
			musthalt = 1;
			continue;
		    }
		    strcpy (prefix, cmde[i+1]);
		    tstamp = prefix + strlen (prefix);
		    i++;
		    continue;
		}
		if (cmde[i][1] == 'p')
		{   
		    if ((prefix = (char *) malloc (strlen (cmde[i] + 2) + 30)) == NULL)
		    {	fprintf (stderr, "could not allocate %d byte for prefix !\n",
			         strlen (cmde[i+1]) + 30);
			musthalt = 1;
			continue;
		    }
		    strcpy (prefix, cmde[i] + 2);
		    tstamp = prefix + strlen (prefix);
		    continue;
		}

		fprintf (stderr, "unknown flag %s\n", cmde[i]);
		musthalt = 1;
	    }
	}

	if (musthalt) exit (-1);    // we must stop because of some bad typo on command line
    }
    
    kdc240_initialize ();

    kdc240_number_of_pictures();
    
    for (id=1 ; id<=number_of_pictures ; id++)
    {   struct Image * image;
	// let's fetch the corresponding picture from the camera...
	image = kdc240_get_picture (id, 0);
	if (image == NULL)
	{   fprintf (stderr, "kdc240_get_picture return NULL at retrieving image no %d (%s)! (skipped)\n",
		     id, picture_index[id-1].filename);
	}
	else
	{   FILE *f;
	    char *p;
	    struct stat statbuf;
	    time_t pt, ct = time (NULL);

	    kdc240_open_card();
	    kdc240_get_picture_info(&picture_index[id-1]);
	    kdc240_close_card();
	    pt = mktime (&(picture_index[id-1].image_time));

	    if (prefix == NULL)
	    {	char *q = strrchr (picture_index[id-1].filename, '\\');
		
		// we keep only the filename part
		if (q == NULL) q = picture_index[id-1].filename; else q++;
		p = (char *) malloc (strlen (q) + 30); 

		sprintf (p, "%s-%04d%02d%02d-%02d%02d%02d.jpg",
			    q,
			    1900 + picture_index[id-1].image_time.tm_year,
			       1 + picture_index[id-1].image_time.tm_mon,
				   picture_index[id-1].image_time.tm_mday,
				   picture_index[id-1].image_time.tm_hour,
				   picture_index[id-1].image_time.tm_min,
				   picture_index[id-1].image_time.tm_sec
			);
	    }
	    else
	    {	
//		struct tm *t = localtime (&lt);
//		sprintf (tstamp, "-%04d%02d%02d-%02d%02d%02d.jpg",
//			    1900 + t->tm_year,
//			       1 + t->tm_mon,
//				   t->tm_mday,
//				   t->tm_hour,
//				   t->tm_min,
//				   t->tm_sec
//			);
		sprintf (tstamp, "-%04d%02d%02d-%02d%02d%02d.jpg",
			    1900 + picture_index[id-1].image_time.tm_year,
			       1 + picture_index[id-1].image_time.tm_mon,
				   picture_index[id-1].image_time.tm_mday,
				   picture_index[id-1].image_time.tm_hour,
				   picture_index[id-1].image_time.tm_min,
				   picture_index[id-1].image_time.tm_sec
			);
		p = prefix;
	    }

	    // and we dump it (it's a jpeg)

	    if (stat (p, &statbuf) == 0)
	    {	int i=1;
		char *ep = p + strlen (p) - 4;	// we erase the ".jpg" extension with our number
		fprintf (stderr, "Warning: a file named \"%s\" already exists.\n", p);
		do
		{   sprintf (ep, "-%03d.jpg", i);
		    i++;
		}   while ((stat (p, &statbuf) == 0) && (i<1000));
		if (stat (p, &statbuf) == 0)
		{   fprintf (stderr, "          could not find a suitable replacement filename up to \"%s\" !!!\n"
			     "          the photo is NOT saved, operation aborted (what's wrong ???)\n\n",
			     p);
		    continue;
		}
		else
		    fprintf (stderr, "        using \"%s\" instead.\n\n", p);

	    }

	    if (pt == (time_t)(-1))
	    {   fprintf (stderr, "Warning: the photo %s seems to have a bad time-stamp !\n"
			 "          You should check the output file and the date settings of your dc240.\n\n",
			 p);
	    }
	    if (difftime (ct, pt) < 0.0)
	    {   fprintf (stderr, "Warning: the photo %s seems to have a time-stamp in the future !\n"
			 "          You should check the output file and the date settings of your dc240.\n\n",
			 p);
	    }
	    if (difftime (ct, pt) > (30*24*60*60.0))
	    {   fprintf (stderr, "Warning: the photo %s seems to have a time-stamp older than one month !\n\n"
			 "         Though this may be normal, You should check the output file and, perhaps,the date settings of your dc240 if something seems wrong.\n",
			 p);
	    }
    
	    f = fopen (p, "w");
	    if (f == NULL)
	    {   fprintf (stderr, "could not open file <%s> for writing (skipped)\n", picture_index[id-1].filename);
	    }
	    else
	    {   fwrite (image->image, 1, image->image_size, f);
		fclose (f);
	    }
	    printf ("%s\n", p);
	    if (prefix == NULL) free (p);
	}
    }
   
    return (0);
}
