
#include <unistd.h>
#include "kodak_dc240_jd.h"
#include <time.h>
#include <ctype.h>

char serial_port[20];

int getidfromname (char * name)
{
    int i;

    kdc240_number_of_pictures();

    for (i = 0; i < number_of_pictures; i++)
	if (strcmp(picture_index[i].filename, name) == 0)
	    return i + 1;

    fprintf (stderr, "Warning: could not find any match in the %d entries for %s\n",
	     number_of_pictures, name);
    return 0;
}

int main (int nb, char ** cmde)
{   int i = 0;
    int nbpic = -1;
    int curpic = 0;
    time_t lt, ct, dt = 10;
    char *prefix = NULL,
         *tstamp = NULL;
    strcpy (serial_port, "/dev/usb/dc240");

    {	int i;
	int musthalt = 0;
	for (i=1 ; i<nb ; i++)
	{
	    if (cmde[i][0] == '-')
	    {	
		if ( (strcmp ("help", cmde[i]+1) == 0) || (strcmp ("h", cmde[i]+1) == 0))
		{   printf ("usage: %s [-d nbsec | -delay nbsec]\n"
			    "       [-p prefix | -prefix prefix]\n"
			    "       [-n nbpic | -nbpic nbpic]\n",
			    cmde[0]);
		    musthalt = 1;
		    continue;
		}

		if ((strcmp ("nbpic", cmde[i]+1) == 0) || (strcmp ("n", cmde[i]+1) == 0))
		{   if (i+1 >= nb)
		    {	fprintf (stderr, "missing data after flag %s\n", cmde[i]);
			musthalt = 1;
			continue;
		    }
		    if (isdigit (cmde[i+1][0]))
		    {   nbpic = atoi (cmde[i+1]);
			i++;
			continue;
		    }
		}
		if ((cmde[i][1] == 'n') && (isdigit (cmde[i][2])))
		{   nbpic = atoi (cmde[i] + 2);
		    continue;
		}

		if ((strcmp ("delay", cmde[i]+1) == 0) || (strcmp ("d", cmde[i]+1) == 0))
		{   if (i+1 >= nb)
		    {	fprintf (stderr, "missing data after flag %s\n", cmde[i]);
			musthalt = 1;
			continue;
		    }
		    if (isdigit (cmde[i+1][0]))
		    {   dt = atoi (cmde[i+1]);
			i++;
			continue;
		    }
		}
		if ((cmde[i][1] == 'd') && (isdigit (cmde[i][2])))
		{   dt = atoi (cmde[i] + 2);
		    continue;
		}

		if ( (strcmp ("prefix", cmde[i]+1) == 0) || (strcmp ("p", cmde[i]+1) == 0))
		{   if (i+1 >= nb)
		    {	fprintf (stderr, "missing data after flag %s\n", cmde[i]);
			musthalt = 1;
			continue;
		    }
		    if ((prefix = (char *) malloc (strlen (cmde[i+1]) + 30)) == NULL)
		    {	fprintf (stderr, "could not allocate %d byte for prefix !\n",
			         strlen (cmde[i+1]) + 30);
			musthalt = 1;
			continue;
		    }
		    strcpy (prefix, cmde[i+1]);
		    tstamp = prefix + strlen (prefix);
		    i++;
		    continue;
		}
		if (cmde[i][1] == 'p')
		{   
		    if ((prefix = (char *) malloc (strlen (cmde[i] + 2) + 30)) == NULL)
		    {	fprintf (stderr, "could not allocate %d byte for prefix !\n",
			         strlen (cmde[i+1]) + 30);
			musthalt = 1;
			continue;
		    }
		    strcpy (prefix, cmde[i] + 2);
		    tstamp = prefix + strlen (prefix);
		    continue;
		}

		fprintf (stderr, "unknown flag %s\n", cmde[i]);
		musthalt = 1;
	    }
	}

	if (musthalt) exit (-1);    // we must stop because of some bad typo on command line

	printf ("starting to shoot ");
	if (nbpic != -1)
	    printf ("%d pictures ", nbpic);
	else
	    printf ("as many pitcures as possible, ");

	printf ("with %d seconds delay between each picture", (int) dt);
	if (prefix == NULL)
	    printf (", using default camera file name for each");
	else
	    printf (", naming every photo name with \"%s\"-YYYYMMDD-hhmmss", prefix);

	if (nbpic == -1)
	    printf (", until (external) program termination");
	
	printf (".\n");
    }
    
    kdc240_initialize ();
    while ((nbpic == -1) || (curpic < nbpic))
    {
	int id;
	PICTURE_FILENAME_PACKET_TYPE buffer = { FALSE, NULL };

	// first, we shot one pic
	lt = time (NULL);
	fprintf (stderr, "prise de vue no %d\n", i);
	kdc240_command(KODAK_CAMERA_DC240, CMD_TAKE_PICTURE);

	ct = time (NULL);
	if (ct - lt > dt)
	    fprintf (stderr, "Warning: taking picture from camera took %ds, "
			     "it's more than the %ds required !\n",
			     (int)(ct - lt), (int)dt);

	// we get its name (the last photo shot)
	{
	    CC_STRUCT_TYPE cc_struct = 
	    {
		(int)&buffer,
		0, NULL,
		256, (void *)kdc240_take_picture_read,
	    };
	    kdc240_complex_command(&cc_struct, KODAK_CAMERA_DC240, CMD_SEND_LAST_PIC);
	}

	// did-it work ?
	if (buffer.valid == FALSE)
	{
	    fprintf (stderr, ": unable to determine filename of last picture!\n");
	}
	else
	{
	    // let's fetch some corresponding gphoto id...
	    id = getidfromname (buffer.filename);
	    if (id == 0)
		fprintf (stderr, "could not retrieve matching filename on DC240 for <%s> ! (skipped)\n",
			 buffer.filename);
	    else
	    {   struct Image * image;
		// let's fetch the corresponding picture from the camera...
		image = kdc240_get_picture (id, 0);
		if (image == NULL)
		{   fprintf (stderr, "kdc240_get_picture return NULL at retrieving image no %d (%s) ! (skipped)\n",
			     id, buffer.filename);
		}
		else
		{   FILE *f;
		    char *p;
		    
		    if (prefix == NULL)
		    {
			p = strrchr (buffer.filename, '\\');
			
			// we keep only the filename part
			if (p == NULL) p = buffer.filename; else p++;
		    }
		    else
		    {	struct tm *t = localtime (&lt);

			sprintf (tstamp, "-%04d%02d%02d-%02d%02d%02d.jpg",
				    1900 + t->tm_year,
				       1 + t->tm_mon,
				           t->tm_mday,
				           t->tm_hour,
				           t->tm_min,
				           t->tm_sec
				);
			p = prefix ;
		    }

		    // and we dump it (it's a jpeg)
		    f = fopen (p, "w");
		    if (f == NULL)
		    {   fprintf (stderr, "could not open file <%s> for writing (skipped)\n", buffer.filename);
		    }
		    else
		    {   fwrite (image->image, 1, image->image_size, f);
			fclose (f);
		    }
		}
	    }

	    // for safetyness we fetch again the gphoto matching id...
	    id = getidfromname (buffer.filename);
	    // fprintf (stderr, "getidfromname returned %d\n", id);
	    if (id == 0)
		fprintf (stderr, "could not retrieve matching filename on DC240 for <%s> ! (skipped)\n",
			 buffer.filename);
	    // if there is only one pic remaining or if it's pic #1 we do not delete..
	    else if (id == 1)
	    {	fprintf (stderr, "deletion of image no 1 (%s) skipped for avoiding lock-up of dc240\n",
			 buffer.filename);
	    }
	    else
	    {   
		// we can try to delete the picture
		if (kdc240_delete_picture (id) != GPHOTO_SUCCESS)
		{	fprintf (stderr, "deletion of image no %d on DC240 (%s) failed (skipped)\n",
			    id, buffer.filename);
		}
		else
		{	fprintf (stderr, "deletion of image no %d on DC240 (%s) done\n\n",
			    id, buffer.filename);
		}
	    }
	}
	free (buffer.filename);

	ct = time (NULL);
	if (ct - lt > dt)
	    fprintf (stderr, "Warning: taking picture from camera and all subsequent operation took %ds, "
			     "it's more than the %ds required !\n",
			     (int)(ct - lt), (int)dt);

	{   time_t it = ct;
	    while (ct - lt < dt)
	    {   // this loop prevents some idle detection from the camera that could shut
		// it down automatically during long shot-intervals
		if (ct - it > 10)
		{
		    kdc240_number_of_pictures();
		    it = ct;
		}
		sleep (1);
		ct = time (NULL);
	    }
	}
	curpic ++;
    }
   
    return (0);
}
